
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,USA
 */

enum
{
	TXT_OTR_MODULE_NAME,
	TXT_OTR_FILL_0,
	TXT_KG_FAILED,
	TXT_KG_COMPLETED,
	TXT_KG_ABORTED_DUP,
	TXT_KG_ABORTED_DIR,
	TXT_KG_MKDIR,
	TXT_KG_PIPE,
	TXT_KG_FORK,
	TXT_KG_INITIATED,
	TXT_KG_EXITED,
	TXT_KG_EXITSIG,
	TXT_KG_POLLERR,
	TXT_KG_ABORT,
	TXT_KG_NEEDACC,
	TXT_KG_NOABORT,
	TXT_KEY_NOT_FOUND,
	TXT_KEY_LOADED,
	TXT_KEY_LOAD_ERROR,
	TXT_OTR_FILL_1,
	TXT_FP_SAVED,
	TXT_FP_SAVE_ERROR,
	TXT_FP_NOT_FOUND,
	TXT_FP_LOADED,
	TXT_FP_LOAD_ERROR,
	TXT_FP_TRUST,
	TXT_OTR_FILL_2,
	TXT_INSTAG_SAVED,
	TXT_INSTAG_SAVE_ERROR,
	TXT_INSTAG_NOT_FOUND,
	TXT_INSTAG_LOADED,
	TXT_INSTAG_LOAD_ERROR,
	TXT_OTR_FILL_3,
	TXT_OPS_NOTIFY_BUG,
	TXT_OPS_NOTIFY,
	TXT_OPS_DISPLAY_BUG,
	TXT_OPS_DISPLAY,
	TXT_OPS_SEC,
	TXT_OPS_FPCOMP,
	TXT_OPS_INSEC,
	TXT_OPS_STILL_REPLY,
	TXT_OPS_STILL_NO_REPLY,
	TXT_OPS_LOG,
	TXT_OPS_INJECT,
	TXT_OPS_HANDLE_MSG,
	TXT_OTR_FILL_4,
	TXT_SEND_FAILED,
	TXT_SEND_CHANGE,
	TXT_SEND_FRAGMENT,
	TXT_SEND_CONVERTED,
	TXT_RECEIVE_IGNORE_QUERY,
	TXT_RECEIVE_DEQUEUED,
	TXT_RECEIVE_QUEUED,
	TXT_RECEIVE_IGNORE,
	TXT_RECEIVE_CONVERTED,
	TXT_OTR_BETTER_TWO,
	TXT_OTR_BETTER_THREE,
	TXT_OTR_FILL_5,
	TXT_CTX_NOT_FOUND,
	TXT_CTX_NOT_CREATE,
	TXT_OTR_FILL_6,
	TXT_AUTH_ABORTED_ONGOING,
	TXT_AUTH_ABORTED,
	TXT_AUTH_RESPONDING,
	TXT_AUTH_INITIATED,
	TXT_AUTH_HAVE_OLD,
	TXT_AUTH_PEER,
	TXT_AUTH_PEER_QA,
	TXT_AUTH_PEER_REPLY_WRONG,
	TXT_AUTH_PEER_REPLIED,
	TXT_AUTH_PEER_WRONG_SMP3,
	TXT_AUTH_PEER_WRONG_SMP4,
	TXT_AUTH_SUCCESSFUL,
	TXT_AUTH_FAILED,
	TXT_AUTH_NEEDENC,
	TXT_OTR_FILL_7,
	TXT_CMD_OTR,
	TXT_CMD_QNOTFOUND,
	TXT_CMD_AUTH,
	TXT_CMD_DEBUG_ON,
	TXT_CMD_DEBUG_OFF,
	TXT_CMD_FINISH,
	TXT_CMD_FINISHALL_NONE,
	TXT_CMD_VERSION,
	TXT_PEER_FINISHED,
	TXT_OTR_FILL_8,
	TXT_CTX_CTX_UNENCRYPTED,
	TXT_CTX_CTX_ENCRYPTED,
	TXT_CTX_CTX_FINISHED,
	TXT_CTX_CTX_UNKNOWN,
	TXT_CTX_FPS_NO,
	TXT_CTX_FPS_SMP,
	TXT_CTX_FPS_MAN,
	TXT_CTX_NOCTXS,
	TXT_OTR_FILL_9,
	TXT_ST_PLAINTEXT,
	TXT_ST_UNTRUSTED,
	TXT_ST_TRUST_SMP,
	TXT_ST_TRUST_MANUAL,
	TXT_ST_SMP_INCOMING,
	TXT_ST_SMP_OUTGOING,
	TXT_ST_SMP_FINALIZE,
	TXT_ST_SMP_UNKNOWN,
	TXT_ST_FINISHED,
	TXT_ST_UNKNOWN
};

extern FORMAT_REC formats[];
